#include <iostream>

using namespace std;

/**
 * A class for storing information about time in military format.
 *
 * @author Mark Maloof (maloof@cs.georgetown.edu)
 * @version 1.0, 8/16/13
 */

class Time
{
  friend ostream& operator<<( ostream& out, const Time& time );
  friend istream& operator>>( istream& in, Time& time );

  public:
    Time();   // default constructor
    Time( const int newHours, const int newMinutes ); // explicit constructor
    int getHours() const; // constant method
    int getMinutes() const;
    void setHours( const int newHours );
    void setMinutes( const int newMinutes );
    void print( ostream& out = cout ) const;
    void println( ostream& out = cout ) const;
    void enter( istream& in = cin );
    void read( istream& in );
    void write( ostream& out ) const;
    void add( const Time& time );
    bool equal( const Time& time ) const;
    bool lessThan( const Time& time ) const;
    void operator+=( const Time& time );
    bool operator==( const Time& time ) const;
    bool operator<( const Time& time ) const;

    /** the maximum value for hours */
    static const int maxHours = 24;

    /** the maximum value for minutes */
    static const int maxMinutes = 60;

    /** a constant for noon */
    static const Time noon;

  private:
    /** the hours */
    int hours;
    /** the minutes */
    int minutes;
};

const Time Time::noon( 12, 0 );

/**
 * Default constructor.  Sets this time object to 00:00.
 */

Time::Time()
{
  hours = 0;
  minutes = 0;
}

/**
 * Explicit constructor.  Sets this time object to the specified hours
 * and minutes.  Out-of-range values will not replace default values.
 *
 * @param newHours the hours of this time object
 * @param newMinutes the minutes of this time object
 */

Time::Time( const int newHours, const int newMinutes )
{
  setHours( 0 );
  setMinutes( 0 );
  setHours( newHours );
  setMinutes( newMinutes );
}

/**
 * Extracts a time in the format "hh:mm" from the specified input
 * stream.  If there are problems with the format of the information in
 * the stream, then no values are set.  Out-of-range values do not replace
 * current values.
 *
 * @param  in  the input stream for reading
 */

void Time::enter( istream& in )
{
  int newHours, newMinutes;
  char ch;
  in >> newHours >> ch >> newMinutes;
  if ( in && ch == ':' ) {
    setHours( newHours );
    setMinutes( newMinutes );
  } // if
}

/**
 * Returns the hours of this time object.
 *
 * @return the hours
 */

int Time::getHours() const
{
  return hours;
}

/**
 * Returns the minutes of this time object.
 *
 * @return the minutes
 */

int Time::getMinutes() const
{
  return minutes;
}

/**
 * Prints this time object to the specified output stream in the
 * format "hh:mm" with no terminating end-of-line character.
 *
 * @param out the output stream for printing
 */

void Time::print( ostream& out ) const
{
  if ( hours < 10 )
    out << '0';
  out << hours;
  out << ':';
  if ( minutes < 10 )
    out << '0';
  out << minutes;
}

/**
 * Prints this time object to the specified output stream in the
 * format "hh:mm" with a terminating end-of-line character.
 *
 * @param out the output stream for printing
 */

void Time::println( ostream& out ) const
{
  print();
  out << endl;
}

/**
 * Reads a time in the format "hh mm" from the specified input stream.
 * If there are problems with the format of the information in the stream,
 * then no values are set.  Out-of-range values do not replace current
 * values.
 *
 * @param  in the specified input stream for reading
 */

void Time::read( istream& in )
{
  int newHours, newMinutes;
  char ch;
  in >> newHours >> newMinutes;
  if ( in  ) {
    setHours( newHours );
    setMinutes( newMinutes );
  } // if
}

/**
 * Sets the hours of this object to the specified hours, which must
 * be in {0,..., 23}.  Out-of-range values do not replace current
 * values.
 *
 * @param  newHours  the specified hours
 */

void Time::setHours( const int newHours )
{
  if ( newHours >= 0 && newHours <= 23 )
    hours = newHours;
}

/**
 * Sets the minutes of this object to the specified minutes, which must
 * be in {0,..., 59}.  Out-of-range values do not replace current
 * values.
 *
 * @param  newMinutes  the specified minutes
 */

void Time::setMinutes( const int newMinutes )
{
  if ( newMinutes >= 0 && newMinutes <= 59 )
    minutes = newMinutes;
}

/**
 * Writes this time object to the specified output stream in the
 * format "hh mm" with no terminating end-of-line character.
 *
 * @param out the output stream for writing
 */

void Time::write( ostream& out ) const
{
  out << hours << ' ' << minutes;
}

/**
 * Adds the specified time object to this time object.
 *
 * @param  time  the time object to be added
 */

void Time::add( const Time& time )
{
  minutes += time.minutes;
  if ( minutes > 59 ) {
    hours += 1;
    minutes = 60 - minutes;
  }
  hours += time.hours;
  if ( hours > 23 )
    hours = 24 - hours;
}

/**
 * Returns true if this time object is equal to the specified time object;
 * returns false otherwise.
 *
 * @param  time  the time object to be compared
 * @return true if the objects are equal
 */

bool Time::equal( const Time& time ) const
{
  return ( hours == time.hours && minutes == time.minutes );
}

/**
 * Returns true if this time object is less than the specified time object;
 * returns false otherwise.
 *
 * @param  time  the time object to be compared
 * @return true if this object is less than the specified object
 */

bool Time::lessThan( const Time& time ) const
{
  if ( hours < time.hours )
    return true;
  if ( hours == time.hours && minutes < time.minutes )
    return true;
  return false;
}

/**
 * Adds the specified time object to this time object.
 *
 * @param  time  the time object to be added
 */

void Time::operator+=( const Time& time )
{
  add( time );
}

/**
 * Returns true if this time object is equal to the specified time object;
 * returns false otherwise.
 *
 * @param  time  the time object to be compared
 * @return true if the objects are equal
 */

bool Time::operator==( const Time& time ) const
{
  return equal( time );
}

/**
 * Returns true if this time object is less than the specified time object;
 * returns false otherwise.
 *
 * @param  time  the time object to be compared
 * @return true if this object is less than the specified object
 */

bool Time::operator<( const Time& time ) const
{
  return lessThan( time );
}

/**
 * Inserts this time object to the specified output stream in the
 * format "hh:mm" with no terminating end-of-line character.
 *
 * @param out the output stream
 * @param time the time object
 */

ostream& operator<<( ostream& out, const Time& time )
{
  time.print( out );
  return out;
}

/**
 * Extracts a time in the format "hh:mm" from the specified input stream
 * and stores it in the specified time object.  If there are problems with
 * the format of the information in the stream, then no values are set.
 * Out-of-range values do not replace current values.
 *
 * @param  in  the input stream for reading
 * @param time the time object
 */

istream& operator>>( istream& in, Time& time )
{
  time.enter( in );
  return in;
}

int main()
{
  cout << Time::maxHours << endl;
  cout << Time::maxMinutes << endl;
	cout << Time::noon << endl;
  return 0;
}
